function [L,Rbasis] = buildLmat(w);  
% function [L,Rbasis] = buildLmat(w);
%
% DESCRIPTION 
% Builds a matrix representation of the linear operator, L:
%      L: Q --> w(x)'*Q*w(x)
% The domain is the set of nxn matrices (not necessarily symmetric) and
% the range space is the set of polynomials.  In constructing L, the 
% basis used for the range space is given by the monomials in the vector 
% Rbasis.  The basis used for the domain is Q_i = reshape(e_i,n,n) where
% e_i is the i^th standard basis vector in R^{n*n}.  
% Thus the construction of L yields:
%            w(x)'*Q*w(x) =   Rbasis'*L*Q(:)
%   
% INPUTS 
%   w:  lw-by-1 vector of monomials 
%
% OUTPUTS 
%   L: lr-by-lw^2 matrix representation of the linear operator L: Q -->
%       w(x)'*Q*w(x). 
%   Rbasis: lr-by-1 vector of monomials which form a basis for the 
%       range space of L.  
%
% SYNTAX 
%   [L,Rbasis] = buildLmat(w);
%
% EXAMPLE
%   pvar x1 x2 
%   w = [x1; x2; x1^2];
%   [L,Rbasis]=buildLmat(w); 
%   Q=rand(3); 
%   w'*Q*w,  Rbasis'*L*Q(:), w'*Q*w-Rbasis'*L*Q(:)
%
% 1/28/08  PJS     Re-coding based on LOCALbuildL

% Error checking
if ~ismonom(w) | size(w,2)~=1
    error('w must be a nw-by-1 vector of monomials');
end
   
if w.nvar ==0
    % Handle case where w(x) = constant
    Rbasis = polynomial(1);
    L = 1;
    return;
else    
    % Build vector of all monomials in range of L, [w'*Q_i*w]
    % XXX This step can cause memory issues since we are constructing
    % large matrices.
    wmat = w.coef'*w.degmat;
    lw = size(wmat,1);
    wQw = repmat(wmat,lw,1) + kron(wmat,ones(lw,1));

    % The basis for the range space is the set of unique monomials in wQw
    [Rbasis,temp,idx] = unique(wQw,'rows');  % Rbasis(idx,:) = wQw
    lr = length(Rbasis);
    chkval = 0; % skip validity check
    Rbasis = polynomial(speye(lr) ,Rbasis, w.var, [lr,1],chkval);

    % Build matrix representation of the linear operator, L:
    %      L: Q --> w'*Q*w
    Qdim = lw^2;
    L = sparse(idx,1:Qdim,1);
end
