function [w,Q0,N,C] = gramsol(p,g,w);  
% function [w,Q0,N,C] = gramsol(p,g,w);  
%
% DESCRIPTION 
%   Construct particular and null solutions for Gram matrix constraint.
%   Given a 1-by-1 polynomial p and a 1-by-N polynomial g then the 
%   following Gram matrix decomposition represents a set of linear 
%   constraints on the unknown coefficients c and the unknown entries of 
%   the matrix Q:
%           p(x)+g(x)*c = w(x)'*Q*w(x)
%   This function computes Q0, Ni and Ci such that:
%         w'*Q0*w = p, w'*Ni*w=0, and w'*Ci*w = gi
%   Thus all solutions of the Gram matrix decomposition are given by:
%         Q0 + sum_i lambda_i N_i + sum_j c_j C_j
%
% INPUTS 
%   p: 1-by-1 polynomial
%   g: 1-by-N polynomial [Optional]
%   w: lw-by-1 monomials vector used in Gram matrix constraint [Optional]
%
% OUTPUTS 
%   w: lw-by-1 monomials vector used in Gram matrix constraint
%   Q0: lw-by-lw matrix satisfying w'*Q0*w = g0
%   N: lw^2-by-M matrix whose i^th column is vec(Ni) where z'*Ni*z = 0.
%   C: lw^2-by-N matrix whose i^th column is vec(Ci) where z'*Ci*z = gi.
%
% SYNTAX 
%   [w,Q0,N] = gramsol(p);  
%   [w,Q0,N,C] = gramsol(p,g);  
%   [w,Q0,N,C] = gramsol(p,g,w);  
%
% EXAMPLE
%   pvar x1 x2 
%   p = 5*x1+3*x1*x2-4*x2^2*x1;
%   g = [x1^2-9, 6*x2*x1];
%   [w,Q0,N,C] = gramsol(p,g);  
%
%   lw = length(w);
%   p - w'*reshape(Q0,lw,lw)*w
%   for i1=1:size(N,2)
%       w'*reshape(N(:,i1),lw,lw)*w
%   end
%   for i1=1:size(C,2)
%       g(i1) - w'*reshape(C(:,i1),lw,lw)*w
%   end

% 2/29/08  PJS     Initial Coding

if nargin < 2
    g = [];
    w = [];
elseif nargin<3
    w = [];
end

%----------------------------------------------------------------------
% Step 1) Build monomials vector needed to write constraint in terms
%   of the the Gram matrix: 
%           p(x) + g(x)*c = w(x)'*Q*w(x)
%   XXX-Prune w(x) with convex hull algo?
%----------------------------------------------------------------------
if ~isempty(w)
    lw = length(w);
else
    h = [p g];
    mxdg = max(h.maxdeg)/2;
    mndg = min(h.mindeg)/2;
    w = monomials( h.nvar , [floor(mndg):ceil(mxdg)]);

    % Discard monomials 
    lw = size(w,1);
    mxdg = ceil(max(h.degmat,[],1)/2);
    [ridx,cidx] = find( repmat(mxdg,lw,1) < w );
    w(ridx,:) = [];

    lw = size(w,1);
    mndg = floor(min(h.degmat,[],1)/2);
    [ridx,cidx] = find( w< repmat(mndg,lw,1) );
    w(ridx,:) = [];

    lw = size(w,1);
    w = polynomial(speye(lw),w,h.var,[lw 1]);
end

%----------------------------------------------------------------------
% Step 2) Build a matrix representation, L, of the linear operator 
%   which maps matrices to polynomials, Q --> w'*Q*w.  L and
%   Rbasis satisfy w'*Q*w=Rbasis'*L*Q(:)
%----------------------------------------------------------------------
[L,Rbasis] = buildLmat(w);  
lr = length(Rbasis);

%----------------------------------------------------------------------
% Step 3) Write p(x) and g(x) in terms of Rbasis:  
%   p(x) = Rbasis'*p_R and g(x) = Rbasis'*g_R
%----------------------------------------------------------------------
p_R = poly2basis(p,Rbasis);
if ~isempty(g)
    g_R = poly2basis(g,Rbasis);
else
    g_R = [];
end

%----------------------------------------------------------------------
% Step 4) p(x)+g(x)*c = w(x)'*Q*w(x) is equivalent to:
%   Rbasis'*(p_R + g_R*c) = Rbasis'*L*Q(:)
% Find symmetric Q0 and Ci such that 
%   p(x) = w(x)'*Q0*w(x) and gi(x) = w(x)'*Ci*w(x)
%----------------------------------------------------------------------
Q0 = L \ p_R;
Q0 = LOCALSymmetrize(Q0);
if ~isempty(g)
    C = L \ g_R;
    C = LOCALSymmetrize(C);
else
    C = [];
end

%----------------------------------------------------------------------
% Step 5) Build a basis for nullspace of L.
%    The columns of N contain vec(Ni) where z'*Ni*z = 0	  
%----------------------------------------------------------------------
N = sparse( lw^2 , 0);
for i1=1:lr
    idx = find(L(i1,:));
    lidx = length(idx);
    if lidx>1
        % Null space vectors can be found row-by-row due to structure of L
        % XXX The non-zero entries of L are all 1 which means L(i1,idx)
        % must always be of the form ones(1,N).  N1compact can then
        % be explicitly given by [ -ones(1,N-1); eye(N-1)]
        %N1compact = null(full(L(i1,idx)),'r');
        N1compact = [-ones(1,lidx-1); eye(lidx-1)];
        N1 = sparse( lw^2 , size(N1compact,2) );
        N1(idx,:) = N1compact;
        
        % Symmetrize Null Space Solutions.  Anti-symmetric solutions are
        % symmetrized to all-zeros and removed.  Then remove non-unique
        % solutions.
        % XXX I think this can be done faster and without the call
        % to unique
        N1 = LOCALSymmetrize(N1);
        idx = find( sum(abs(N1))==0 );  
        N1(:,idx) = [];
        N1 = unique(N1','rows')';        
        N = [N N1];
    end
end

%----------------------------------------------------------------------
% function Mout = LOCALSymmetrize(Min);
%   Min is an n^2-by-m matrix with the k^th column representing a
%   vectorized n-by-n matrix, Mk.  Mout is an n^2-by-m matrix with 
%   the k^th column representing the symmetric form of Mk.
%----------------------------------------------------------------------
function Mout = LOCALSymmetrize(Min);

[nsq,m]=size(Min);
n = sqrt(nsq);
Mout = sparse(nsq,m);
for i1=1:m
    Mk = reshape(Min(:,i1),n,n);
    Mk = (Mk+Mk')/2;
    Mout(:,i1) = Mk(:);
end
