function [feas,z,Q,f] = issos(p,opts)
% function [feas,z,Q,f] = issos(p,opts)
%
% DESCRIPTION 
%   This function tests if a polynomial is a sum of squares. It returns
%   the Gram matrix decomposition and the SOS decomposition.
%
% INPUTS 
%   p:  1-by-1 polynomial
%   opts: Structure of options with the following fields:
%       -solver: Optimization solver to be used. Choices are 'sedumi',
%          'sdpam', 'csdp', 'dsdp', or 'setup'.  solver='setup' will
%          simply set up the problem without solving the SDP.
%          Default is 'sedumi'.
%       -form: Formulation for the optimization. Choices are 'image'
%         (sedumi primal form) or 'kernel' (sedumi dual form). 
%         Default is 'image'.
%       -scaling: Scaling of SOS constraints. Choices are 'on' or 'off'.
%         Default is 'off'.
%       -checkfeas: Check feasibility of solution.  Choices are 'off',
%         'fast' 'full', and 'both'. 'fast' checks feasibility information 
%         returned by the solver. 'full' checks the validity of the Gram 
%         matrix decomposition in the output sossol. 'both' does both 
%         feasibility checks. sossol and dopt are returned as empty if the 
%         optmiization solution is infeasible. Default is 'fast'.
%       -The remaining fields of opts are passed directly to the solver.
%         Default is to set the solver options to turn off display.
%       
% OUTPUTS
%   feas: feas=1 if the polynomial is a SOS and =0 otherwise.
%   z: lz-by-1 vector of monomials in the Gram matrix decomposition of p, 
%        if it exists. z is returned as empty if feas = 0.
%   Q: lz-by-lz positive semidefinite matrix in the SOS decomposition 
%        of p, if it exists. Q is returned as empty if feas = 0.
%   f: lf-by-1 vector of polynomials in the SOS decomposition of p, if
%        it exists.  In this case p = f(1)^2 + f(2)^2 + ... + f(end)^2.
%        f is returned as empty if feas = 0.
%
% SYNTAX 
%   [feas,z,Q,f] = issos(p)
%
% EXAMPLE 
%  pvar x1 x2;
%  p = 2*x1^4 + 2*x1^3*x2 - x1^2*x2^2 + 5*x2^4;
%  [feas,z,Q,f] = issos(p)
%  
%  % Verify Gram matrix decomposition
%  p - z'*Q*z
%  % Verify Q >=0
%  min(eig(Q))
%  % Verify SOS decomposition of p
%  p - f'*f
%
% See also: sosopt

% 6/3/09   PJS     Initial Coding

% Setup
if nargin==1
    opts = [];
end

% Promote to poly and handle constant case
p = polynomial(p);
if p.nvar==0
   p = double(p);
   if p>=0
        feas = 1;
        z = polynomial(1);
        Q = p;
        f = polynomial(sqrt(p));
        return       
   else
        feas = 0;
        z = [];
        Q = [];
        f = [];
        return
   end
end

% Simply call sosopt and unpack
[info,dopt,sossol] = sosopt(p,opts);
feas = info.feas;
if feas==1
    z = sossol{2};
    Q = sossol{3};
    
    % Construct SOS decomposition 
    % Q = U*T*U' --> Q = L'*L where L = sqrt(T)*U'
    [U,T] = schur(Q);  
    T = max(T,0);  % This should only be zeroing tiny negative evals
    L = sqrt(T)*U';
    f = L*z; 
else
    z = [];
    Q = [];
    f = [];
end
