function [x,y,solverinfo] = solvesossdp(sdpdata,opts); 
% function [x,y,solverinfo] = solvesossdp(sdpdata,opts);
%
% DESCRIPTION 
%   This function solves a cone optimization specified in Sedumi format
%   using the specified by solver.  Solver can be 'dsdp', 'csdp' 
%   'sdpam', 'sedumi', 'setup' (Default is 'sedumi')
%
% INPUTS 
%   sdp: Structure describing the cone optimization problem in Sedumi
%       format.  The fields are A,b,c,K.  See Sedumi 
%       documentation for more details on the format.
%   opts: Structure of options with the following fields:
%       -solver: Optimization solver to be used. Choices are 'sedumi',
%          'sdpam', 'csdp', 'dsdp', or 'setup'.  solver='setup' will
%          simply return empty solutions. (Default is 'sedumi'.)
%       -The remaining fields of opts are passed directly to the solver.
%         Default is to set the solver options to turn off display.
%       
% OUTPUTS
%   x,y: Optimization results in Sedumi output format. See Sedumi
%       documentation.
%   solverinfo: Info returned by solver.  See documentation for the 
%       specified solver.
%
% SYNTAX 
%   [x,y,solverinfo] = solvesossdp(sdpdata);
%   [x,y,solverinfo] = solvesossdp(sdpdata,opts);
%
% EXAMPLE
%

% 5/5/09  PJS  Initial Coding

% Get sdp problem data
A=sdpdata.A;
b=sdpdata.b;
c=sdpdata.c;
K=sdpdata.K;

% Set default solver
if nargin<2
    solver = 'sedumi';
elseif isfield(opts,'solver')
    solver = opts.solver;
    if isempty(solver)
        solver = 'sedumi';
    end
else
    solver = 'sedumi';
end
    
% Solve SDP
if strcmpi(solver,'dsdp')
    [x,y,solverinfo] = sedumi2dsdp(A,b,c,K,opts); 
elseif strcmpi(solver,'csdp')
    [x,y,solverinfo] = sedumi2csdp(A,b,c,K,opts);   
elseif strcmpi(solver,'sdpam')
    [x,y,solverinfo] = sedumi2sdpam(A,b,c,K,opts);   
elseif strcmpi(solver,'sedumi')
    if ~isfield(opts,'eps')
        opts.eps = 1e-9; 
    end    
    if ~isfield(opts,'fid')
        opts.fid = 0; 
    end    
    [x,y,solverinfo] = sedumi(A,b,c,K,opts); 
elseif strcmpi(solver,'setup')
    x=[]; y=[]; solverinfo=[];
else
    error(['Solver ' solver ' is not available']);
end
