function  [info,dopt,sossol] = sosopt(sosconstr,x,obj,lpconstr,opts)
% function [info,dopt,sossol] = sosopt(sosconstr,x,obj,lpconstr,opts)
%
% DESCRIPTION 
%   This function solves an optimization or feasibility problem with 
%   linear programming and sum-of-squares constraints.  The optimization 
%   has of the form:
%           min w'*d
%           subject to:   si(x,d) is sum of squares  (i=1,...,Ns)
%                         A*d<=b                   
%   d is a vector of decision variables. Refer to the polydecvar
%   function for the creation of polynomial decision variables.
%
% INPUTS 
%   sosconstr: Ns-by-1 cell array of polynomials.  These polynomials
%       will be constrained to be sums-of-squares.  They must be linear
%       functions of the decision variables.
%   x: Nx-by-1 array of pvars.  These are the independent variables in
%       the optimization.  All remaining variables will be decision
%       variables.
%   obj: 1-by-1 polynomial. obj must be a linear function of the decision
%       variables. Set obj =[] for a feasibility problem
%   lpconstr: Structure with fields A,b,d which describe the LP 
%       constraint. lpconstr.d can be a subset of the decision variables. 
%   opts: Structure of options with the following fields:
%       -solver: Optimization solver to be used. Choices are 'sedumi',
%          'sdpam', 'csdp', 'dsdp', or 'setup'.  solver='setup' will
%          simply set up the problem without solving the SDP.
%          Default is 'sedumi'.
%       -form: Formulation for the optimization. Choices are 'image'
%         (sedumi primal form) or 'kernel' (sedumi dual form). 
%         Default is 'image'.
%       -scaling: Scaling of SOS constraints. Choices are 'on' or 'off'.
%         Default is 'off'.
%       -checkfeas: Check feasibility of solution.  Choices are 'off',
%         'fast' 'full', and 'both'. 'fast' checks feasibility information 
%         returned by the solver. 'full' checks the validity of the Gram 
%         matrix decomposition in the output sossol. 'both' does both 
%         feasibility checks. sossol and dopt are returned as empty if the 
%         optmiization solution is infeasible. Default is 'fast'.
%       -feastol: Feasibility tolerance used in full feasibility check.
%         Default is feastol = 1e-6.
%       -The remaining fields of opts are passed directly to the solver.
%         Default is to set the solver options to turn off display.
%       
% OUTPUTS
%   info: Struture with fields: 
%       - feas: feas=1 if the SOS optimization is feasible and feas=0
%           otherwise.
%       - opts: Options structure.
%       - sosdata: Structure with fields sosconstr, x, decvars, obj,
%           lpconstr, scaling.  This is the SOS optimization information.
%       - sos2sdp: Structure with fields w,dv2x,dv2y,Nfv,Nlv,orderidx. 
%           This is information used internally to relate the decision
%           variables / constraints in the SOS program to the variables/
%           constraints in the SDP optimization.
%       - sdpdata: Structure with fields A,b,c,K.  This is the SDP
%           optimization problem specified in Sedumi form. 
%       - sdpsol: Structure with fields x, y, solverinfo.  x,y is the
%           solution returned in Sedumi format. solverinfo is any info
%           returned by the specified solver.
%   dopt: Nd-by-2 cell array of the optimal decision variables.  The 
%       first column of dopt contains the decision variables (as chars)
%       and the second column contains the optimal values.  The subs 
%       command can be used to replace the decision variables in any 
%       polynomial with their optimal values.  For example, the minimal 
%       value of the cost function can be computed by 
%               subs(obj,dopt(:,1),dopt(:,2)).
%   sossol: Ns-by-3 cell array with columns: constr, w, and Q. constr
%       is the sosconstr polynomial evaluated at the optimial decision
%       variables, i.e. 
%               sossol{i,1} = subs(sosconstr{i},dopt(:,1),dopt(:,2))
%       w=sossol{i,2} is a vector of monomials in x and Q=sossol{i,3} is 
%       a positive semi-definite matrix such that sosconstr{i} = w'*Q*w.
%       This is a Gram matrix decomposition which proves constr is SOS.
%
% SYNTAX 
% % Same functionality as issos
%  [info,dopt,sossol] = sosopt(p)
% % Solve a feasibility problem with or without SOS/Solver options
%  [info,dopt,sossol] = sosopt(sosconstr,x);
%  [info,dopt,sossol] = sosopt(sosconstr,x,opts);
% % Solve a SOS minimization with or without SOS/Solver options
%  [info,dopt,sossol] = sosopt(sosconstr,x,obj);
%  [info,dopt,sossol] = sosopt(sosconstr,x,obj,opts);
% % Solve a SOS minimization with LP constraints with or without options
%  [info,dopt,sossol] = sosopt(sosconstr,x,obj,lpconstr);
%  [info,dopt,sossol] = sosopt(sosconstr,x,obj,lpconstr,opts);
% % To re-solve without repeating the SOS-to-SDP conversion:
%  [info,dopt,sossol]=sosopt(info)
%
% See also sosp, sosd, checkfeas, polydecvar

% 3/05/08  PJS     Initial Coding
% 4/25/09  PJS     Cleaning up documentation + SOS dec var handling
% 5/6/09 PJS       Modify to allow set-up call


%---------------------------------------------------------------------
% Error checking
%---------------------------------------------------------------------

if nargin==1 && isstruct(sosconstr)
    % First argument is info structure containing problem set-up data
    sosdata = sosconstr;
    opts = sosconstr.opts;
    
else
    %---------------------------------------------------------------------
    % Process Inputs
    %---------------------------------------------------------------------
    
    if nargin < 2
        x = []; obj = []; lpconstr = []; opts = []; 
    elseif nargin < 3
        if isa(x,'struct')
            opts = x;
            x = []; obj = []; lpconstr = []; 
        else
            obj = []; lpconstr = []; opts = [];
        end
    elseif nargin <4
        if isa(obj,'struct')
            opts= obj;
            obj = []; lpconstr = [];
        else
            lpconstr = []; opts = [];
        end
    elseif nargin <5 
        if isa(lpconstr,'struct')
            opts = lpconstr;
            lpconstr = [];
        else
            opts = []; 
        end
    end

    % Convert polynomial sosconstr to a cell array 
    if isa(sosconstr,'polynomial');
        Ns = length(sosconstr(:));
        tmp = cell(Ns,1);
        for i1=1:Ns
            tmp{i1} = sosconstr(i1);
        end
        sosconstr = tmp;
    end

    % Default Options
    if ~isfield(opts,'scaling') || isempty(opts.scaling)
        opts.scaling = 'off';
    end
    if ~isfield(opts,'form') || isempty(opts.form)
        opts.form = 'image';
    end
    if ~isfield(opts,'solver') || isempty(opts.solver)
        opts.solver = 'sedumi'; 
    end
    if ~isfield(opts,'checkfeas') || isempty(opts.checkfeas)
        opts.checkfeas = 'fast'; 
    end
    if ~isfield(opts,'feastol') || isempty(opts.feastol)
        opts.feastol = 1e-6; 
    end

    %---------------------------------------------------------------------
    % Find decision variables 
    %---------------------------------------------------------------------
    Nsc =length(sosconstr);
    if Nsc==0
        error('Optimization must include at least one SOS constraint');
    end
    allvar = [];
    for i1=1:Nsc
        s = sosconstr{i1};
        s = polynomial(s);
        allvar = [allvar; s.varname];   
    end

    if isempty(x) 
        Nx = length(allvar);
        x = polynomial(speye(Nx),speye(Nx),allvar,[Nx 1]);
    end

    if ~isempty(lpconstr)
        dlp = lpconstr.d;
        if ~ispvar(dlp) || length(dlp)~=dlp.nvar
            error('lpconstr.d must be a vector of unique pvars');
        end
        allvar = [allvar; dlp.var];
    end

    if iscellstr(x)
        decvars = setdiff(allvar,x);    
    else
        decvars = setdiff(allvar,x.var);
    end

    %---------------------------------------------------------------------
    % Find SOS decision variables
    %---------------------------------------------------------------------
    Nsos = 0;
    for i1=1:Nsc    
        s = sosconstr{i1};
        if isa(s,'polydecvar')
            if strcmp(s.type,'mat') || ispvar(s)
                % This constraint is a pure SOS dec variable. 
                sosconstr{i1} = {polynomial(s),s.monomials,s.decvars};
                Nsos = Nsos+1;
            end
        end        
    end
    if Nsos==Nsc
        % SDP set-up requires at least one non-pure SOS constraint
        sosconstr{1} = sosconstr{1}{1};
    end
    

    %---------------------------------------------------------------------
    % Scale SOS Constraints
    %---------------------------------------------------------------------
    if strcmpi(opts.scaling,'on');
        scl = ones(Nsc,1);
        for i1=1:Nsc
            s = sosconstr{i1};
            if isa(s,'polynomial')
                scl(i1) = sqrt(norm(s.coefficient));
                s.coefficient = s.coefficient/scl(i1);
                sosconstr{i1} = s;
            end
        end
    else
        scl = ones(Nsc,1);
    end
    
    %---------------------------------------------------------------------    
    % Store all data about the SOS problem formulation 
    %---------------------------------------------------------------------
    sosdata.sosconstr = sosconstr;
    sosdata.x = x;
    sosdata.decvars = decvars;
    sosdata.obj = obj;
    sosdata.lpconstr = lpconstr;
    sosdata.scaling = scl;
    
    info.sosdata = sosdata;
    info.opts = opts;
end

%---------------------------------------------------------------------
% Call Image/Kernel Forms of SOS Optimization
%---------------------------------------------------------------------
if (nargout==3) || any( strcmpi(opts.checkfeas,{'full';'both'}) )
    if strcmpi(opts.form,'image');
        [info,dopt,sossol]=sosp(sosdata,opts);
    elseif strcmpi(opts.form,'kernel');
        [info,dopt,sossol]=sosd(sosdata,opts);
    else
        error([opts.form ' form is not available']);
    end    
else    
    if strcmpi(opts.form,'image');
        [info,dopt]=sosp(sosdata,opts);
    elseif strcmpi(opts.form,'kernel');
        [info,dopt]=sosd(sosdata,opts);
    else
        error([opts.form ' form is not available']);
    end
end

%---------------------------------------------------------------------
% Check feasibility of solution
%---------------------------------------------------------------------
if strcmpi(info.opts.solver,'setup')
    feas = 0;
elseif strcmpi(opts.checkfeas,'off');
    feas = 1;
elseif strcmpi(opts.checkfeas,'full');    
    feas = checkfeas(sossol,opts.feastol);    
elseif strcmpi(opts.checkfeas,'both');    
    feas = checkfeas(info,sossol,opts.feastol);
elseif strcmpi(opts.checkfeas,'fast');      
    feas = checkfeas(info);
else
    error('checkfeas option must be: off, full, both, or fast');
end
info.feas = feas;
info = orderfields(info,{'feas','opts','sosdata','sos2sdp','sdpdata','sdpsol'});

if feas==0
   % Infeasible solution
    sossol = [];
    dopt = [];
end