function  [info,dopt,sossol] = sosp(sosdata,opts)
% function [info,dopt,sossol] = sosp(sosdata,opts)
%
% DESCRIPTION 
%   This function solves the SOS optimization or feasibility problem in
%   image (primal) form.  An SOS optimization contains constraints of the
%   form (see sosopt for more details):
%               s(x,d) is sum of squares
%   where d are the decision variables and x are the polynomial variables.
%
%   This function solves the optimization by introducing a matrix decision
%   variable Q for each SOS constraint and rewriting the constraint as:
%               s(x,d)=w(x)'*Q*w(x)
%               Q>=0
%   The constraint s(x,d)=w(x)'*Q*w(x) is then represented as a linear 
%   equality constraint between the entries of Q, the decision variables 
%   d, and the numeric coefficients of s.
%
% See also sosopt, sosd

% 1/29/08  PJS     Initial Coding
% 2/26/08  PJS     Added interface to DSDP
% 11/11/08 PJS     Separately handle pure sos variables
% 4/25/09 PJS      Clean up documentation and code for pure sos vars
% 5/6/09 PJS       Modify to allow set-up call
% 5/29/09 PJS      Modify to handle symmetric SOS coef matrix

%---------------------------------------------------------------------
% Formulate SOS Optimization as an SDP
%---------------------------------------------------------------------
if isfield(sosdata,'sdpdata')
    % Input includes all SOS/SDP problem set-up information
    info = sosdata;
    sosdata = info.sosdata;
    sdpdata = info.sdpdata;
    sos2sdp = info.sos2sdp;
    opts = info.opts;

    % Get additional data needed to compute dopt and sossol
    sosconstr=sosdata.sosconstr;
    Nsc =length(sosconstr);
    decvars = sosdata.decvars;
    Ndv = length(decvars);
    scaling = sosdata.scaling;
    
    w = sos2sdp.w;
    dv2xs = sos2sdp.dv2x;
    Nfv = sos2sdp.Nfv;
    Nlv = sos2sdp.Nlv;
    orderidx = sos2sdp.orderidx;
    
    K = sdpdata.K;
else
    % Get data about SOS problem formulation
    sosconstr=sosdata.sosconstr;
    x=sosdata.x;
    decvars = sosdata.decvars;
    obj = sosdata.obj;
    lpconstr = sosdata.lpconstr;
    scaling = sosdata.scaling;

    %---------------------------------------------------------------------
    % Find sos inequalities and sos variable constraints. 
    % Reorder to have sos var constraints precede sos ineq.
    % Compute indexing, dv2xs, of decision vars into sedumi vars, xs 
    % (see def of xs below)
    %---------------------------------------------------------------------
    Nsc =length(sosconstr);
    sosineqidx = [];
    sosvaridx = [];
    sosvars = [];
    dv2xs = [];
    Nsv = 0;
    for i1=1:Nsc
        s = sosconstr{i1};
        if isa(s,'polynomial')
            % SOS inequality constraint
            sosineqidx = [sosineqidx, i1];
        else 
            % SOS variable constraint
            sosvaridx = [sosvaridx, i1];
            
            % C is symmetric.  Grab dec vars from lower half of C
            C = s{3}; 
            lz = size(C,1);
            idx = find(tril(ones(lz)));            
            sosvars = [sosvars; char(C(idx))];
            
            % Index mapping from symmetric decvars to non-symmetric
            dv2xs = [dv2xs Nsv+idx(:)'];
            Nsv = Nsv+lz^2;
        end
    end
    orderidx = [sosvaridx sosineqidx];

    % Find number of free dec vars (Nfv), SOS non-symmetric matrix vars 
    % (Nsv), and LP slack vars (Nlv), total decision vars (Ndv)
    freevars = setdiff(decvars,sosvars);
    Nfv = length(freevars);
    if ~isempty(lpconstr)
        Nlv = length(lpconstr.b);
    else
        Nlv = 0;
    end
    dv2xs = [1:Nfv  dv2xs+Nfv+Nlv];
    decvars = [freevars; sosvars];
    Ndv = length(decvars);

    %---------------------------------------------------------------------
    % Define SOS/LP Cost and Constraints in Sedumi Primal Form
    %       min cs'*xs   s.t. As*xs = bs  ;  xs >= 0
    %
    % SOS Constraints: The constraint s(x,d) is SOS is equivalent to the 
    %   constraints s(x,d) = w(x)'*Q*w(x) and Q>=0.  This can be expressed
    %   as Ad*d + Aq*Q(:) = b and Q>=0 for appropriately defined vector b 
    %   and matrices Ad and Aq. Q is a PSD slack matrix.
    % LP Constraints: A*d<=b is defined as A*d+y=b and y>=0 where 
    %   y is a slack variable.
    % SOS Variables: As described above, generic SOS constraints are
    %   represented as a set of equality constraints with a slack PSD
    %   matrix.  SOS variables do not require the introduction of a slack
    %   matrix.  They are directly reresented as w(x)'*M*w(x) where M is a
    %   non-symmetric matrix of variables [due to Sedumi format) and w(x) 
    %   is a vector of monomials. This constraint can be directly 
    %   represented as M>=0. SOS variables are represented using a
    %   symmetric matrix of decision variables. These decision vars map
    %   into the lower half of the Sedumi matrix vars.
    %
    % The order of variables in the sedumi format is free variables, LP
    % variables, and semidefinite variables. For the SOS program, this 
    % ordering is:
    %   xs = [Free decision vars (Nfv-by-1);
    %         y = LP slack vars  (Nlv-by-1);
    %         M(:) = SOS dec vars in the form of PSD matrices (Nsv-by-1); 
    %         Q(:) = PSD slack matrices for SOS inequalities]
    %---------------------------------------------------------------------

    % Define Number of Free Decision Variables
    K.f = Nfv;

    % Define LP constraints: A*d+y=b, y>=0
    As = sparse([]); 
    bs = sparse([]);
    K.l = [];
    if ~isempty(lpconstr)
        % Define LP constraint dimension
        A = lpconstr.A;
        b = lpconstr.b;
        d = lpconstr.d; 
        Nconstr = length(b);
        K.l = [K.l Nconstr];

        % Find the decision variables involved in the LP constraint
        [isdv,dvidx]=ismember(char(d),decvars);
        if ~all( isdv )
            error('LP constraints must only contain decision variables');
        end

        % Stack A*d+y=b into As*xs=bs where the free dec. vars fv precede
        % the slack variables, y, which precede the SOS dec vars sv.
        tmp = sparse(Nconstr,Nfv+Nlv+Nsv);
        tmp(:,dv2xs(dvidx)) = A;
        tmp(:,Nfv+(1:Nconstr)) = speye(Nconstr);  
        As = tmp;
        bs = [bs; b];    
    end
    K.l = sum(K.l);  % Group all LP constraints

    % Define SOS Variable Constraints: M>=0
    Nsosvarc = length(sosvaridx);
    ws = cell(Nsosvarc,1);
    K.s = [];
    for i1=1:Nsosvarc
        s = sosconstr{ sosvaridx(i1) };
        ws{i1} = s{2};
        K.s = [K.s length(s{2})];
    end

    % Define SOS Inequality Constraints: Ad*d + Aq*Q(:) = b and Q>=0
    Nsosineqc = length(sosineqidx);
    wi = cell(Nsosineqc,1);
    ptr = Nfv+Nlv+Nsv;
    for i1=1:length(sosineqidx)
        s = sosconstr{ sosineqidx(i1) };

        % Collect s(x,d) into the form g0(x)+g1(x)*d1 + ... + gN(x)*dN
        % where di are decision variables.
        [g0,g,d] = collect(s,x);
        if ispvar(d)
            % Find the decision variables involved in the SOS constraint
            [isdec,didx]=ismember(char(d),decvars);            
        elseif d==0
            isdec=0; 
            didx=[];
        else
            error('SOS constraint must be linear in the decision variables');
        end

        % Express g0(x)+g(x)*d is SOS as 
        %     g0(x)+g(x)*d = w(x)'*Q*w(x),  Q>=0
        % Then convert the polynomial equality constraint into:
        %           Ad*d + Aq*Q(:) = b    
        if any(isdec)
            [w,b,Aq,Ad] = gramconstraint(g0,g);  
            Ad = -Ad;
        else
            [w,b,Aq] = gramconstraint(g0);          
        end
        wi{i1} = w;
        Nconstr = length(b);

        % Stack Ad*d + Aq*Q(:) = b into As*xs=bs where the psd slack vars,
        % Q(:), follow the free dec vars fv, the LP slack vars y, and the
        % SOS dec vars sv.
        K.s = [K.s length(w)];              
        tmp = [sparse(Nconstr,ptr) Aq];
        if any(isdec)
            tmp(:,dv2xs(didx)) = Ad;
        end
        if isempty(As)
            As = tmp;
        else        
            As = [ [As sparse(size(As,1),size(Aq,2))] ; tmp]; 
        end
        bs = [bs; b];    
        ptr = size(As,2);    
    end
    w = [ws;wi];

    %---------------------------------------------------------------------
    % Create the objective function vector
    %---------------------------------------------------------------------

    if isempty(obj)
        % Feasibility Problem
        cs = sparse( size(As,2) , 1 );
    else
        % Objective function is obj = cs'*xs    
        [g0,g,d] = collect(obj,x);
        if (~ispvar(d) && d~=0) ||  (g0.nvar ~=0) || (g.nvar~=0)
            error('SOS constraint must be linear in the decision variables');
        end
        g0 = double(g0);
        [isdec,didx]=ismember(char(d),decvars);    
        cs = sparse( size(As,2) , 1 );
        cs(dv2xs(didx)) = double(g);
    end  

    %---------------------------------------------------------------------
    % Create info data structure 
    %---------------------------------------------------------------------

    % sosdata
    sosdata.decvars = decvars;
    
    % sdpdata
    sdpdata.A = As;
    sdpdata.b = bs;
    sdpdata.c = cs;
    sdpdata.K = K;

    % sos2sdp: w, dv2x, dv2y, Nfv, Nlv, orderidx
    sos2sdp.w = w;
    sos2sdp.dv2x = dv2xs;
    sos2sdp.dv2y = [];
    sos2sdp.Nfv = Nfv;
    sos2sdp.Nlv = Nlv;
    sos2sdp.orderidx = orderidx;

    % Pack all data into info
    info.sosdata = sosdata;
    info.sdpdata = sdpdata;
    info.sos2sdp = sos2sdp;
    info.opts = opts;
end

%---------------------------------------------------------------------
% Solve the problem 
%---------------------------------------------------------------------
if strcmpi(opts.solver,'setup')
    % Return with problem formulation for setup call
    sdpsol.x = [];
    sdpsol.y = [];
    sdpsol.solverinfo = [];
    info.sdpsol = sdpsol;
    
    dopt =[];
    sossol = [];
    return;
end
[xs,ys,solverinfo] = solvesossdp(sdpdata,opts); 

% sdpsol: x,y, solverinfo
sdpsol.x = xs;
sdpsol.y = ys;
sdpsol.solverinfo = solverinfo;
info.sdpsol = sdpsol;

%---------------------------------------------------------------------
% Create sossol and dopt for output
%---------------------------------------------------------------------

% Create dopt
if Ndv ~= 0
    dopt = [decvars(:) num2cell(xs(dv2xs))];
else
    dopt = [];
end

if nargout==3
    % Create sossol 
    % The call to subs is only moderately fast so only do this if needed
    cnt = Nfv+Nlv;
    sossol = cell(Nsc,3);

    for i1 = 1:Nsc
        idx = orderidx(i1);
        s = sosconstr{idx};
        if ~isa(s,'polynomial')
            % SOS variable constraint
            s = s{1};
        end        
        if Ndv~=0
            s = subs(s,dopt); 
        end
        xidx = cnt+(1:K.s(i1)^2);
        Q = reshape( xs(xidx), K.s(i1), K.s(i1) ); 
       
        % Unscale solution
        scl = scaling(idx);
        if scl~=1
            s.coefficient = s.coefficient*scl;
            Q = Q*scl;
        end       
        
        % Store Gram Matrix solution for output
        sossol{idx,1} = s;
        sossol{idx,2} = w{i1};   
        sossol{idx,3} = Q;        
        
        cnt = xidx(end);
    end
end        
